88cf7dbf45
* Add hipify steps prior to build
105 lines
3.7 KiB
Plaintext
105 lines
3.7 KiB
Plaintext
/*************************************************************************
|
|
* Copyright (c) 2016-2022, NVIDIA CORPORATION. All rights reserved.
|
|
* Modifications Copyright (c) 2019-2022 Advanced Micro Devices, Inc. All rights reserved.
|
|
*
|
|
* See LICENSE.txt for license information
|
|
************************************************************************/
|
|
|
|
#include "cuda_runtime.h"
|
|
#include "common.h"
|
|
|
|
#define ALIGN 4
|
|
|
|
void ReduceScatterGetCollByteCount(size_t *sendcount, size_t *recvcount, size_t *paramcount, size_t *sendInplaceOffset, size_t *recvInplaceOffset, size_t count, int nranks) {
|
|
size_t base = (count/(ALIGN*nranks))*ALIGN;
|
|
*sendcount = base*nranks;
|
|
*recvcount = base;
|
|
*sendInplaceOffset = 0;
|
|
*recvInplaceOffset = base;
|
|
*paramcount = base;
|
|
}
|
|
|
|
testResult_t ReduceScatterInitData(struct threadArgs* args, ncclDataType_t type, ncclRedOp_t op, int root, int rep, int in_place) {
|
|
size_t sendcount = args->sendBytes / wordSize(type);
|
|
size_t recvcount = args->expectedBytes / wordSize(type);
|
|
int nranks = args->nProcs*args->nThreads*args->nGpus;
|
|
|
|
for (int i=0; i<args->nGpus; i++) {
|
|
CUDACHECK(cudaSetDevice(args->gpus[i]));
|
|
int rank = ((args->proc*args->nThreads + args->thread)*args->nGpus + i);
|
|
CUDACHECK(cudaMemset(args->recvbuffs[i], 0, args->expectedBytes));
|
|
void* data = in_place ? args->recvbuffs[i] : args->sendbuffs[i];
|
|
TESTCHECK(InitData(data, sendcount, 0, type, op, rep, nranks, rank));
|
|
CUDACHECK(cudaMemcpy(args->expected[i], args->recvbuffs[i], args->expectedBytes, cudaMemcpyDefault));
|
|
TESTCHECK(InitDataReduce(args->expected[i], recvcount, rank*recvcount, type, op, rep, nranks));
|
|
CUDACHECK(cudaDeviceSynchronize());
|
|
}
|
|
return testSuccess;
|
|
}
|
|
|
|
void ReduceScatterGetBw(size_t count, int typesize, double sec, double* algBw, double* busBw, int nranks) {
|
|
double baseBw = (double)(count * typesize * nranks) / 1.0E9 / sec;
|
|
|
|
*algBw = baseBw;
|
|
double factor = ((double)(nranks - 1))/((double)nranks);
|
|
*busBw = baseBw * factor;
|
|
}
|
|
|
|
testResult_t ReduceScatterRunColl(void* sendbuff, void* recvbuff, size_t count, ncclDataType_t type, ncclRedOp_t op, int root, ncclComm_t comm, cudaStream_t stream) {
|
|
NCCLCHECK(ncclReduceScatter(sendbuff, recvbuff, count, type, op, comm, stream));
|
|
return testSuccess;
|
|
}
|
|
|
|
struct testColl reduceScatterTest = {
|
|
"ReduceScatter",
|
|
ReduceScatterGetCollByteCount,
|
|
ReduceScatterInitData,
|
|
ReduceScatterGetBw,
|
|
ReduceScatterRunColl
|
|
};
|
|
|
|
void ReduceScatterGetBuffSize(size_t *sendcount, size_t *recvcount, size_t count, int nranks) {
|
|
size_t paramcount, sendInplaceOffset, recvInplaceOffset;
|
|
ReduceScatterGetCollByteCount(sendcount, recvcount, ¶mcount, &sendInplaceOffset, &recvInplaceOffset, count, nranks);
|
|
}
|
|
|
|
testResult_t ReduceScatterRunTest(struct threadArgs* args, int root, ncclDataType_t type, const char* typeName, ncclRedOp_t op, const char* opName) {
|
|
args->collTest = &reduceScatterTest;
|
|
ncclDataType_t *run_types;
|
|
ncclRedOp_t *run_ops;
|
|
const char **run_typenames, **run_opnames;
|
|
int type_count, op_count;
|
|
|
|
if ((int)type != -1) {
|
|
type_count = 1;
|
|
run_types = &type;
|
|
run_typenames = &typeName;
|
|
} else {
|
|
type_count = test_typenum;
|
|
run_types = test_types;
|
|
run_typenames = test_typenames;
|
|
}
|
|
|
|
if ((int)op != -1) {
|
|
run_ops = &op;
|
|
run_opnames = &opName;
|
|
op_count = 1;
|
|
} else {
|
|
op_count = test_opnum;
|
|
run_ops = test_ops;
|
|
run_opnames = test_opnames;
|
|
}
|
|
|
|
for (int i=0; i<type_count; i++) {
|
|
for (int j=0; j<op_count; j++) {
|
|
TESTCHECK(TimeTest(args, run_types[i], run_typenames[i], run_ops[j], run_opnames[j], -1));
|
|
}
|
|
}
|
|
return testSuccess;
|
|
}
|
|
|
|
struct testEngine ncclTestEngine = {
|
|
ReduceScatterGetBuffSize,
|
|
ReduceScatterRunTest
|
|
};
|